#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
text2pdf.py - a script to convert text files to pdf. iterates through a directory, loads each file, and converts it to ONE pdf.

python text2pdf.py -i /path/to/input/dir -o /path/to/output/dir will create one pdf from all txt files in the input directory and save it to the output directory. Add -r to load files recursively.

"""
import sys
import argparse
import pprint as pp
from pathlib import Path

from tqdm.auto import tqdm

from confectionary.pdf import PDF
from confectionary.report_generation import (
    estimate_TOC_pages,
    render_toc,
)
from confectionary.utils import (
    cleantxt_wrap,
    get_seq2replace,
    get_timestamp,
    load_files_ext,
    simple_rename,
    get_user,
    print_api_info,
)


def str_to_pdf(
    text: str,
    output_dir=None,
    key_phrase: str = None,
    create_ewriter_notes=False,
    nltk_usepunkt=True,
    do_paragraph_splitting=True,
    word2vec_model="glove-wiki-gigaword-100",
    be_verbose=False,
):
    """
    str_to_pdf - the most basic version, creates a PDF with no table of contents with a string as input.

    Parameters
    ----------
    text : str, required, the text to be converted to PDF
    output_dir : str, optional, the directory to write the output PDF to. Defaults to None, which will write to the current working directory.
    key_phrase : str, optional, the key phrase to be used to identify the file. Defaults to None, which will use the timestamp.
    create_ewriter_notes : bool, optional, whether to create ewriter notes. Defaults to False.
    nltk_usepunkt : bool, optional, whether to use nltk punkt tokenizer. Defaults to True.
    do_paragraph_splitting : bool, optional, whether to split the text into paragraphs. Defaults to True.
    word2vec_model : str, optional, the word2vec model to use. Defaults to "glove-wiki-gigaword-100".
    be_verbose : bool, optional, whether to print verbose output. Defaults to False.

    Returns
    -------
    pathlib.Path, the path to the output PDF file.
    """
    output_dir = Path.cwd() if output_dir is None else Path(output_dir)
    key_phrase = (
        f"Confectionary text2pdf - {get_timestamp()}"
        if key_phrase is None
        else key_phrase
    )
    pdf = PDF(
        orientation="P",
        unit="mm",
        format="A4",
        is_ewriter=create_ewriter_notes,
        key_phrase=key_phrase,
        split_paragraphs=do_paragraph_splitting,
    )
    title = f"{key_phrase}"
    pdf.set_title(title)
    pdf.set_author(get_user())

    pdf.update_margins()  # update formatting
    pdf.update_title_formats()

    pdf.add_page()

    pdf.write_big_title(title)
    pdf.generic_text(
        text,
        split_paragraphs=do_paragraph_splitting,
        word2vec_model=word2vec_model,
        use_punkt=nltk_usepunkt,
    )

    doc_margin_type = "ewriter" if create_ewriter_notes else "standard"
    pdf_name = (
        f"{key_phrase}_txt2pdf_{get_timestamp(detailed=True)}_{doc_margin_type}.pdf"
    )
    pdf.output(output_dir / pdf_name)

    _out = output_dir / pdf_name
    if be_verbose:
        print(f"\nPDF file written to {_out}")
    return _out


def file_to_pdf(
    filepath,
    output_dir=None,
    key_phrase: str = None,
    intro_text: str = None,
    create_ewriter_notes=False,
    nltk_usepunkt=True,
    do_paragraph_splitting=True,
    word2vec_model="glove-wiki-gigaword-100",
    keywords_enabled=True,
    be_verbose=False,
):
    """
    file_to_pdf - create a pdf from a single text file.

    Parameters
    ----------
    filepath : str, required, the path to the file to be converted to PDF.
    output_dir : str, optional, the directory to write the output PDF to. Defaults to None, which will write to the current working directory.
    key_phrase : str, optional, the key phrase to be used to identify the file. Defaults to None, which will use the filename.
    intro_text : str, optional, the text to be added to the beginning of the file. Defaults to None, which will not add any text.
    create_ewriter_notes : bool, optional, whether to create ewriter notes. Defaults to False.
    nltk_usepunkt : bool, optional, whether to use nltk punkt tokenizer. Defaults to True.
    do_paragraph_splitting : bool, optional, whether to split the text into paragraphs. Defaults to True.
    word2vec_model : str, optional, the word2vec model to use. Defaults to "glove-wiki-gigaword-100".
    be_verbose : bool, optional, whether to print verbose output. Defaults to False.

    Returns
    -------
    pathlib.Path, the path to the output PDF file.
    """
    src_path = Path(filepath)
    output_dir = src_path.parent if output_dir is None else Path(output_dir)
    key_phrase = f"{src_path.stem} - text2pdf" if key_phrase is None else key_phrase
    pdf = PDF(
        orientation="P",
        unit="mm",
        format="A4",
        is_ewriter=create_ewriter_notes,
        key_phrase=key_phrase,
        split_paragraphs=do_paragraph_splitting,
        keywords_enabled=keywords_enabled,
    )
    title = f"{key_phrase}"
    pdf.set_title(title)
    pdf.set_author(get_user())

    pdf.update_margins()  # update formatting
    pdf.update_title_formats()

    if intro_text is not None:
        pdf.add_page()
        pdf.comment_text(intro_text, preamble="")
    if be_verbose:
        print(f"attempting to print {src_path.name}")
    pdf.print_chapter(
        filepath=str(src_path.resolve()),
        num=1,
        title=key_phrase,
        word2vec_model=word2vec_model,
        use_punkt=nltk_usepunkt,
    )
    # save the generated file
    doc_margin_type = "ewriter" if create_ewriter_notes else "standard"
    pdf_name = (
        f"{key_phrase}_txt2pdf_{get_timestamp(detailed=True)}_{doc_margin_type}.pdf"
    )
    pdf.output(output_dir / pdf_name)

    _out = output_dir / pdf_name
    if be_verbose:
        print(f"\nPDF file written to {_out}")
    return _out


def dir_to_pdf(
    input_dir,
    output_dir=None,
    extension: str = ".txt",
    recurse=False,
    key_phrase: str = None,
    intro_text: str = None,
    toc_comments: str = None,
    create_ewriter_notes=False,
    nltk_usepunkt=True,
    do_paragraph_splitting=True,
    word2vec_model="glove-wiki-gigaword-100",
    keywords_enabled=True,
    be_verbose=False,
):
    """
    dir_to_pdf - converts all files in a directory 'input_dir' with extension 'extension' to a single pdf.

    Parameters
    ----------
    input_dir : str or pathlib.Path, the path to the directory containing the files to convert into a single PDF.
    output_dir : str or pathlib.Path, optional, the path to the directory to write the output files to. Defaults to input_dir
    extension : str, optional, the extension of the files to convert when iterating through the directory. Defaults to ".txt".
    recurse : bool, optional, whether to load files recursively from the input directory. Defaults to False
    key_phrase : str, optional, the key phrase to identify the conversion instance. Defaults to None
    intro_text : str, optional, the text to be written at the top of the output file. Defaults to None
    toc_comments : str, optional, the text to be written at the bottom of the output file. Defaults to None
    create_ewriter_notes : bool, optional, whether to write the output to ewriter format (narrow text width). Defaults to False
    nltk_usepunkt : bool, optional, whether to use nltk punkt tokenizer. Defaults to True
    do_paragraph_splitting : bool, optional, whether to split the text into paragraphs. Defaults to True
    word2vec_model : str, optional, the word2vec model to use. Defaults to "glove-wiki-gigaword-100".
    be_verbose : bool, optional, whether to print verbose output. Defaults to False

    Returns
    -------
    pathlib.Path, the path to the output file
    """

    src_dir = Path(input_dir)
    out_dir = Path(output_dir) if output_dir else src_dir
    key_phrase = "Confectionary txt2pdf" if key_phrase is None else key_phrase

    out_p_full = out_dir / "text-to-PDF"
    out_p_full.mkdir(parents=True, exist_ok=True)

    approved_files = load_files_ext(
        src_dir, ext=extension, recursive=recurse, verbose=be_verbose
    )
    assert len(approved_files) > 0, "No files found in the input directory"
    if be_verbose:
        pp.pprint([f.name for f in approved_files])
    print(f"{len(approved_files)} files found matching extension {extension}")

    pdf = PDF(
        orientation="P",
        unit="mm",
        format="A4",
        is_ewriter=create_ewriter_notes,
        key_phrase=key_phrase,
        split_paragraphs=do_paragraph_splitting,
        keywords_enabled=keywords_enabled,
    )
    title = f"{key_phrase}"
    pdf.set_title(title)
    pdf.set_author(get_user())

    pdf.update_margins()  # update formatting
    pdf.update_title_formats()

    pdf.add_page()

    pdf.write_big_title(title)

    # write comments
    if intro_text is None:
        intro_text = f"The below text was loaded from a folder. \
        The files are sorted alphabetically. The fname of the parent folder is:\n\t{src_dir.name}"
    if toc_comments is None:
        toc_comments = """
        Click on chapters in the TOC to be linked there.
        Click on the page footer (bottom right on any page) to return to the TOC.
        """
    pdf.comment_text(cleantxt_wrap(intro_text), 10)
    pdf.ln(1)
    pdf.comment_text(toc_comments, 14)

    # add a table of contents
    toc_pages = estimate_TOC_pages(len(approved_files), verbose=True)
    pdf.insert_toc_placeholder(render_toc, pages=toc_pages)

    seq2replace = get_seq2replace()  # define words to replace in the chapter names

    if be_verbose:
        print("\nPrinting Chapters to PDF")
    pbar = tqdm(total=len(approved_files), desc="Building Chapters in PDF file")
    for i, textfile in enumerate(approved_files):
        out_name = simple_rename(textfile, max_char_orig=75, no_ext=True).lower()
        for ugly_w in seq2replace:
            out_name = out_name.replace(ugly_w, "")  # clean up filename
        out_name = out_name[0].upper() + out_name[1:]  # capitalize first letter
        if be_verbose:
            print(f"attempting chapter {i} - filename: {out_name}")
        pdf.print_chapter(
            filepath=str(textfile.resolve()),
            num=i,
            title=out_name,
            word2vec_model=word2vec_model,
            use_punkt=nltk_usepunkt,
        )
        pbar.update()
    pbar.close()

    if be_verbose:
        print("\nPrinting TOC to PDF")

    # save the generated file
    doc_margin_type = "ewriter" if create_ewriter_notes else "standard"
    pdf_name = f"{key_phrase}_txt2pdf_{get_timestamp()}_{doc_margin_type}.pdf"
    pdf.output(out_p_full / pdf_name)

    _out = out_p_full / pdf_name
    if be_verbose:
        print(f"\nPDF file written to {_out}")
    return _out


def get_parser():
    """
    get_parser - a helper function for the argparse module
    """
    parser = argparse.ArgumentParser(description="Spell correct and create PDF")
    parser.add_argument(
        "-i",
        "--input-dir",
        required=False,
        default=None,
        type=str,
        help="path to directory containing input files. Required if not using the --api-info flag",
    )
    parser.add_argument(
        "-o",
        "--output-dir",
        required=False,
        default=None,
        type=str,
        help="path to directory to write output files (new folder created). Defaults to input-dir",
    )
    parser.add_argument(
        "-kw",
        "--keywords",
        required=False,
        default=None,
        type=str,
        help="some key words or phrases to help identify the output file",
    )
    parser.add_argument(
        "-e",
        "--ewriter-notes",
        required=False,
        default=False,
        action="store_true",
        help="if set, will write the output to ewriter format (narrow text width)",
    )
    parser.add_argument(
        "--no-split",
        required=False,
        default=False,
        action="store_true",
        help="if set, will not split the text into paragraphs (faster, less readable)",
    )
    parser.add_argument(
        "-m",
        "--model",
        required=False,
        default="glove-wiki-gigaword-100",
        type=str,
        help="the word2vec model to use",
    )
    parser.add_argument(
        "--no-punkt",
        required=False,
        default=False,
        action="store_true",
        help="if set, will not use nltk punkt tokenizer",
    )
    parser.add_argument(
        "-r",
        "--recursive",
        required=False,
        default=False,
        action="store_true",
        help="whether to load files recursively from the input directory",
    )
    parser.add_argument(
        "--no-keywords",
        default=False,
        action="store_true",
        help="if set, will not add keywords to the PDF",
    )
    parser.add_argument(
        "--api-info",
        required=False,
        default=False,
        action="store_true",
        help="print the available word2vec models in the gensim API and exit",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        required=False,
        default=False,
        action="store_true",
        help="whether to print verbose output",
    )

    return parser


if __name__ == "__main__":

    args = get_parser().parse_args()
    be_verbose = args.verbose

    if args.api_info:
        print_api_info(verbose=be_verbose)
        sys.exit("Exiting...")
    if args.input_dir is None:
        sys.exit(
            "No input directory specified. Pass an input directory with the -i flag."
        )
    input_dir = args.input_dir
    output_dir = args.output_dir
    key_phrase = args.keywords
    create_ewriter_notes = args.ewriter_notes
    do_paragraph_splitting = not args.no_split
    word2vec_model = args.model
    nltk_usepunkt = not args.no_punkt
    recurse = args.recursive
    keywords_enabled = not args.no_keywords
    _finished_pdf_loc = dir_to_pdf(
        input_dir=input_dir,
        output_dir=output_dir,
        key_phrase=key_phrase,
        create_ewriter_notes=create_ewriter_notes,
        do_paragraph_splitting=do_paragraph_splitting,
        word2vec_model=word2vec_model,
        nltk_usepunkt=nltk_usepunkt,
        be_verbose=be_verbose,
        recurse=recurse,
        keywords_enabled=keywords_enabled,
    )

    print(f"\nPDF file written to {_finished_pdf_loc}")



"""
This file contains utility functions for the project.
"""

import os
import pprint as pp
import random
import re
import shutil
import warnings
from datetime import datetime
from os.path import join
from pathlib import Path

import gensim.downloader as api
import wordninja
import yake
from cleantext import clean
from natsort import natsorted


def get_user():
    """get_user - returns the username of the user running the script"""
    try:
        return os.getlogin()
    except Exception:
        return "unknown"


def get_seq2replace(additional_terms: list = None):
    """
    get_seq2replace - returns a list of terms to be replaced from text. This is used to remove "filename" stopwords from the text.

    Parameters
    ----------
    additional_terms : list, optional, default None, list of additional terms to be replaced

    Returns
    -------
    list, list of terms to be replaced
    """
    seq2replace = [
        "fins",
        "phones",
        "cons",
        "fin",
        "pegasus",
        "phone",
        "con",
        "-v-",
        "---",
        "summ",
        "sum",
        "ocr",
    ]
    if additional_terms is not None:
        seq2replace.extend(additional_terms)
    return seq2replace


def get_first_number(my_string: str):
    """
    get_first_number - return the first number found in a string with regex as an integer

    Parameters
    ----------
    my_string : str, required, input

    Returns
    -------
    int, first number found in the string
    """
    search = re.search(r"\d+", my_string)
    if search is None:
        warnings.warn(
            "No number found in string: {}, returning random".format(my_string)
        )
        return random.randint(0, 10**8)
    else:
        return int(search.group(0))


def fix_punct_spaces(string: str):
    """
    fix_punct_spaces - replace spaces around punctuation with punctuation. For example, "hello , there" -> "hello, there"

    Parameters
    ----------
    string : str, required, input string to be corrected

    Returns
    -------
    str, corrected string
    """

    fix_spaces = re.compile(r"\s*([?!.,]+(?:\s+[?!.,]+)*)\s*")
    string = fix_spaces.sub(lambda x: "{} ".format(x.group(1).replace(" ", "")), string)
    string = string.replace(" ' ", "'")
    string = string.replace(' " ', '"')
    string = string.replace("- _", "-")
    string = string.replace("_ -", "-")
    string = string.replace(" _ ", "-")
    string = string.replace("_ ", "-")
    string = string.strip("_")
    return string.strip()


def dict_sort_by_keys(d: dict):
    """
    dict_sort_by_keys - given a dictionary, sorts and returns the dictionary sorted by ascending keys

    Parameters
    ----------
    d : dict, required, input dictionary

    Returns
    -------
    dict, sorted dictionary
    """
    return {k: v for k, v in natsorted(d.items(), key=lambda item: item[0])}


def dict_sort_by_vals(d: dict):
    """
    dict_sort_by_keys - given a dictionary, sorts and returns the dictionary sorted by ascending values

    Parameters
    ----------
    d : dict, required, input dictionary

    Returns
    -------
    dict, sorted dictionary
    """
    return {k: v for k, v in natsorted(d.items(), key=lambda item: item[1])}


def load_files_ext(target_dir, ext: str, recursive=False, verbose=False):
    """
    load_files_ext - loads all files with a given extension from a directory, optionally recursively. It then returns a list of the files that is sorted by name.

    :param path: Path to the files.
    :param ext: Extension of the files. e.g. 'txt'
    :param recursive: If True, the function will recursively search for files.

    :return: List of files, sorted by the first number found in the filename.
    """
    target_dir = Path(target_dir)
    ext = ext.replace(".", "")  # make sure extension is lowercase and without dot
    files = []
    if verbose:
        print("Loading files from {} with extension {}".format(target_dir, ext))
    if recursive:
        for file in target_dir.glob("**/*." + ext):
            files.append(file)
    else:
        for file in target_dir.glob("*." + ext):
            files.append(file)
    ids = [get_first_number(file.name) for file in files]
    my_dict = dict(zip(files, ids))
    sorted_dict = dict_sort_by_vals(my_dict)
    return sorted_dict.keys()


def get_timestamp(detailed=False):
    """
    get_timestamp - returns a timestamp in string format

    detailed : bool, optional, default False, if True, returns a timestamp with seconds
    """
    if detailed:
        return datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    else:
        return datetime.now().strftime("%b-%d-%Y")


def create_folder(directory):
    os.makedirs(directory, exist_ok=True)


def corr(s: str):
    """
    corr - adds space after period if there isn't one. For example, "hello.there" -> "hello. there"
            removes extra spaces. For example, "hello  there" -> "hello there"
    """
    return re.sub(r"\.(?! )", ". ", re.sub(r" +", " ", s))


def cleantxt_wrap(ugly_text: str):
    # a wrapper for clean text with options different than default

    # https://pypi.org/project/clean-text/
    cleaned_text = clean(
        ugly_text,
        fix_unicode=True,  # fix various unicode errors
        to_ascii=True,  # transliterate to closest ASCII representation
        lower=False,  # lowercase text
        no_line_breaks=True,  # fully strip line breaks as opposed to only normalizing them
        no_urls=True,  # replace all URLs with a special token
        no_emails=True,  # replace all email addresses with a special token
        no_phone_numbers=True,  # replace all phone numbers with a special token
        no_numbers=False,  # replace all numbers with a special token
        no_digits=False,  # replace all digits with a special token
        no_currency_symbols=True,  # replace all currency symbols with a special token
        no_punct=True,  # remove punctuations
        replace_with_punct="",  # instead of removing punctuations you may replace them
        replace_with_url="<URL>",
        replace_with_email="<EMAIL>",
        replace_with_phone_number="<PHONE>",
        replace_with_number="<NUM>",
        replace_with_digit="0",
        replace_with_currency_symbol="<CUR>",
        lang="en",  # set to 'de' for German special handling
    )

    return cleaned_text


def cleantxt_summary(ugly_text, lang="en"):
    # a wrapper for clean text with options different than default

    # https://pypi.org/project/clean-text/
    cleaned_text = clean(
        ugly_text,
        fix_unicode=True,  # fix various unicode errors
        to_ascii=True,  # transliterate to closest ASCII representation
        lower=False,  # lowercase text
        no_line_breaks=True,  # fully strip line breaks as opposed to only normalizing them
        no_urls=True,  # replace all URLs with a special token
        no_emails=True,  # replace all email addresses with a special token
        no_phone_numbers=True,  # replace all phone numbers with a special token
        no_numbers=False,  # replace all numbers with a special token
        no_digits=False,  # replace all digits with a special token
        no_currency_symbols=True,  # replace all currency symbols with a special token
        no_punct=False,  # remove punctuations
        replace_with_punct="",  # instead of removing punctuations you may replace them
        replace_with_url="<URL>",
        replace_with_email="<EMAIL>",
        replace_with_phone_number="<PHONE>",
        replace_with_number="<NUM>",
        replace_with_digit="0",
        replace_with_currency_symbol="<CUR>",
        lang=lang,  # de
    )

    return cleaned_text


def beautify_filename(
    filename,
    num_words=20,
    start_reverse=False,
    word_separator="_",
    replace_underscores=True,
):
    """
    beautify_filename - given a filename, returns a string with the first num_words words separated by word_separator

    Parameters
    ----------
    filename : str, required, input filename
    num_words : int, optional, number of words to include in the string, default is 20
    start_reverse : bool, optional, if True, the first num_words words are reversed, default is False
    word_separator : str, optional, word separator, default is "_"
    replace_underscores : bool, optional, if True, replaces underscores with a whitespace at the end, default is True

    Returns
    -------
    str, string with the first num_words words separated by word_separator
    """

    filename = str(filename)
    current_name = filename.split(".")[0]
    extension = filename.split(".")[-1]
    if current_name.isnumeric():
        current_name = current_name + extension
        return current_name
    clean_name = cleantxt_wrap(current_name)
    file_words = wordninja.split(clean_name)
    # splits concatenated text into a list of words based on common word freq

    if num_words > len(file_words):
        t_file_words = file_words
    else:
        if start_reverse:
            t_file_words = file_words[-num_words:]
        else:
            t_file_words = file_words[:num_words]

    pretty_name = word_separator.join(t_file_words)  # see function argument
    pretty_name = pretty_name.replace("_", " ") if replace_underscores else pretty_name
    # NOTE IT DOES NOT RETURN THE EXTENSION
    return pretty_name.strip()


def simple_rename(
    filepath,
    header: str = "",
    max_char_orig: int = None,
    target_ext: str = ".txt",
    no_ext: bool = False,
):
    """
    simple_rename - given a filepath, extracts the base name and adds a header, and returns a new filepath with extension target_ext. The extracted name is truncated to max_char_orig if specified.

    Parameters
    ----------
    filepath : str or pathlib.Path, required, input filepath
    header : str, optional, header to add to the file name, default is ""
    max_char_orig : int, optional, maximum number of characters to keep from the original name, default is None (no truncation)
    target_ext : str, optional, target extension, default is ".txt"

    Returns
    -------
    str, new file name with extension target_ext (if no_ext is False)
    """
    _fp = Path(filepath)
    basename = _fp.stem
    max_char_orig = len(basename) if max_char_orig is None else max_char_orig
    target_ext = target_ext if target_ext.startswith(".") else f".{target_ext}"

    new_name = (
        f"{header}_{basename[:max_char_orig]}{target_ext}"
        if not no_ext
        else f"{header}{basename[:max_char_orig]}"
    )
    renamed = fix_punct_spaces(new_name)

    return renamed.strip()


def move2completed(from_dir, filename, new_folder="completed", verbose=False):
    """
    move2completed moves a file from a directory to a new directory.

    Parameters
    ----------
    from_dir : str, the directory to move the file from
    filename : str, the name of the file to move
    new_folder : str, optional, the name of the new directory to move the file to, defaults to "completed"
    verbose : bool, optional, whether to print the file moved, defaults to False
    """

    # this is the better version
    old_filepath = join(from_dir, filename)

    new_filedirectory = join(from_dir, new_folder)

    if not os.path.isdir(new_filedirectory):
        os.mkdir(new_filedirectory)
        if verbose:
            print("created new directory for files at: \n", new_filedirectory)

    new_filepath = join(new_filedirectory, filename)

    try:
        shutil.move(old_filepath, new_filepath)
        print("successfully moved the file {} to */completed.".format(filename))
    except:
        print(
            "ERROR! unable to move file to \n{}. Please investigate".format(
                new_filepath
            )
        )


def digest_txt_directory(src_dir, identifer="", verbose=False, make_folder=False):
    """
    digest_txt_directory - merge all text files together in a given directory

    Parameters
    ----------
    src_dir : str
        the directory to digest
    identifer : str
        the identifier to use for the digest file
    verbose : bool
        if true, prints out the digest file name
    make_folder : bool
        if true, creates a folder for the digest file

    Returns
    -------
    digest_file : str
        the digest file name
    """

    digest_file = identifer + "_digest.txt"
    digest_file_path = join(src_dir, digest_file)

    if make_folder:
        digest_folder = identifer + "_digest"
        digest_folder_path = join(src_dir, digest_folder)
        if not os.path.isdir(digest_folder_path):
            os.mkdir(digest_folder_path)

    with open(digest_file_path, "w") as f:
        for filename in os.listdir(src_dir):
            if filename.endswith(".txt"):
                with open(join(src_dir, filename), "r") as f2:
                    f.write(f2.read())
                    f.write("\n\n")

    if verbose:
        print("created digest file at: \n", digest_file_path)

    return digest_file_path


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def create_kw_extractor(
    language="en",
    max_ngram_size=3,
    deduplication_algo="seqm",
    windowSize=10,
    numOfKeywords=10,
    ddpt=0.7,
):
    """
    creates a keyword extractor object

    :param language: language of the text
    :param max_ngram_size: max ngram size
    :param deduplication_algo: deduplication algorithm
    :param windowSize: window size
    :param numOfKeywords: number of keywords
    :param ddpt: Deduplication Percentage Threshold

    :return: keyword extractor object
    """
    assert ddpt >= 0 and ddpt <= 1, f"need 0<thresh<1, got {ddpt}"
    return yake.KeywordExtractor(
        lan=language,
        n=max_ngram_size,
        dedupLim=ddpt,
        dedupFunc=deduplication_algo,
        windowsSize=windowSize,
        top=numOfKeywords,
        features=None,
    )


def find_text_keywords(body_text: str, yake_ex=None, return_list=False, verbose=False):
    """
    find_text_keywords - find keywords in a text using YAKE

    Parameters
    ----------
    body_text : str, required, the text to find keywords in
    yake_ex : _type_, optional, keyword extractor object, default is None and will create a new one
    verbose : bool, optional, whether to print the keywords, default is False
    return_list : bool, optional, whether to return a list of keywords, default is False

    Returns
    -------
    keywords : str, the keywords found in the text as a string
    """
    yake_ex = yake_ex or create_kw_extractor()
    keywords = yake_ex.extract_keywords(body_text)
    kw_list = ["{} - {}".format(keywords.index(kw), kw[0]) for kw in keywords]
    if return_list:
        return kw_list
    kw_string = ", ".join(kw_list)
    if verbose:
        print(f"found {len(keywords)} keywords: \n{kw_string}")
    return kw_string


def print_api_info(verbose=False):
    """print_api_info - prints possible word2vec models to load for paragraph splitting"""

    model_info = api.info()["models"]

    labels = [m for m in model_info.keys()]
    print("Available word2vec models:")
    if verbose:
        pp.pprint(model_info)
        print("See https://github.com/RaRe-Technologies/gensim-data for more info")
    else:
        pp.pprint(labels)
